// Copyright 2015 CoreOS, Inc.
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package timeutil

import (
	"reflect"
	"testing"
	"time"
)

func TestUnixNanoToTime(t *testing.T) {
	tests := []struct {
		ns   int64
		want time.Time
	}{
		{
			0,
			time.Time{},
		},
		{
			60000,
			time.Unix(0, 60000),
		},
		{
			-60000,
			time.Unix(0, -60000),
		},
	}

	for i, tt := range tests {
		got := UnixNanoToTime(tt.ns)
		if !reflect.DeepEqual(got, tt.want) {
			t.Errorf("#%d: time = %v, want %v", i, got, tt.want)
		}
	}
}
