package walpb;

import "code.google.com/p/gogoprotobuf/gogoproto/gogo.proto";

option (gogoproto.marshaler_all) = true;
option (gogoproto.sizer_all) = true;
option (gogoproto.unmarshaler_all) = true;
option (gogoproto.goproto_getters_all) = false;

message Record {
	required int64 type  = 1 [(gogoproto.nullable) = false];
	required uint32 crc  = 2 [(gogoproto.nullable) = false];
	optional bytes data  = 3;
}

message Snapshot {
	required uint64 index = 1 [(gogoproto.nullable) = false];
	required uint64 term  = 2 [(gogoproto.nullable) = false];
}
