// Copyright 2015 CoreOS, Inc.
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package main

import (
	"flag"
	"log"

	"github.com/coreos/etcd/migrate"
)

func main() {
	from := flag.String("data-dir", "", "etcd v0.4 data-dir")
	name := flag.String("name", "", "etcd node name")
	flag.Parse()

	if *from == "" {
		log.Fatal("Must provide -data-dir flag")
	}

	err := migrate.Migrate4To2(*from, *name)
	if err != nil {
		log.Fatalf("Failed migrating data-dir: %v", err)
	}
}
